require 'uu/os/cmd/command_client'

module UU
  module OS
    module CMD

      # Client for asynchronous command executor.
      # @private
      class CommandExecutor # :nodoc:
        include UU::OS::Util::URLConverter

        # Command executor service path.
        PATH = '/ues/platform/command/UESAsyncCommandExecutor'

        def initialize(session, options = nil)
          opts = {}
          opts.merge!(options) if !options.nil?
          opts[:default_action] = ''
          opts[:main_object_uri_parameter_name] = :uesuri
          opts[:use_legacy_config] = true
          opts[:append_routing_data] = false
          cfg = UU::OS::CMD::CommandClient.const_get(:EXTERNAL_CONFIGURATION)
          opts[:base_url] = cfg.get('server-url', PATH) if opts[:base_url].nil? 
          path_prefix = cfg.get('server-path-prefix', PATH)
          @client = UU::OS::CMD::CommandClient.new(concat_path(path_prefix, PATH), session, opts)
        end

        # Invokes asynchronous command.
        # @param [URI, String] main_object_uri URI of main object.
        # @param [Hash] parameters Invocation parameters:
        # @option parameters [URI, String] :queue_uri Queue where to plan invocation.
        # @option parameters [String] :app_url_code Application URL code in format "+vendor+-+app+[-+module+]".
        # @option parameters [String] :cmd_url_path URL path of command to be invoked.
        # @option parameters [Object] :session Instance of session under which to
        #    invoke asynchronous command.
        # @option parameters [Hash] :parameters Asynchronous invocation parameters
        # @option parameters [Hash] :callback Callback parameters.
        # @return [URI] Asynchronous task URI.
        def invoke(main_object_uri, parameters = {})
          raise ArgumentError, 'queue_uri is not set' if parameters[:queue_uri].nil?
          raise ArgumentError, 'app_url_code is not set' if parameters[:app_url_code].nil?
          raise ArgumentError, 'cmd_url_path is not set' if parameters[:cmd_url_path].nil?

          params = {}
          params[:mainObjectUri] = main_object_uri
          params[:appUrlCode] = parameters[:app_url_code]
          params[:cmdUrlPath] = parameters[:cmd_url_path]
          params[:action] = parameters[:action] || :exec
          if parameters[:session].kind_of?Hash
            params[:authToken] =  parameters[:session][:accessToken]
          elsif parameters[:session].respond_to?(:access_token)
            params[:authToken] =  parameters[:session].access_token
          elsif parameters[:session].kind_of?(String)
            params[:authToken] =  parameters[:session]
          end
          params[:parameters] = parameters[:parameters]
          params[:callback] = parameters[:callback]
          return @client.invoke(:invoke, parameters[:queue_uri], parameters: params)
        end

        # Returns state of asynchronous invocation.
        # @param [URI, String] task_uri Asynchronous task URI.
        # @return [Hash] Asynchronous task state.
        def get_state(task_uri)
          return @client.invoke(:getState, task_uri, invocation_method: :get)
        end

        # Returns base URL for which is this instance of client configured.
        # @return [String] Client base URL.
        # @private
        def base_url # :nodoc:
          return @client.base_url
        end

        # Override inspect to hide internal structures.
        # @private
        def inspect # :nodoc:
          "#<#{self.class.name}:0x#{(self.object_id << 1).to_s(16)}>"
        end

      end

    end
  end
end
